@@ -21,7 +21,6 @@ class NoseGAE(Plugin):
     restricted when running under GAE.
     """
     name = 'gae'
-    lib_dirs = ('.', 'lib/django', 'lib/webob', 'lib/yaml/lib', 'lib/antlr3')
     
     def options(self, parser, env=os.environ):
         super(NoseGAE, self).options(parser, env)
@@ -40,6 +39,11 @@ class NoseGAE(Plugin):
             help='Set the path to the GAE datastore to use in tests. '
             'Note that when using an existing datastore directory, the '
             'datastore will not be cleared before testing begins.')
+        parser.add_option(
+            '--without-sandbox', default=True, action='store_false', dest='sandbox_enabled',
+            help='Enable this flag if you want to run your tests without ' 
+            'import module sandbox. This is most useful when you have a '
+            'conflicting nose plugin (such as coverage).')
 
     def configure(self, options, config):
         super(NoseGAE, self).configure(options, config)
@@ -52,8 +56,7 @@ class NoseGAE(Plugin):
             self._path = config.workingDir
         if options.gae_lib_root is not None:
             root = self._gae_path = options.gae_lib_root
-            for d in self.lib_dirs:
-                sys.path.insert(0, os.path.join(root, d))
+            sys.path.append(root)
         else:
             self._gae_path = None
         if options.gae_data is not None:
@@ -63,7 +66,21 @@ class NoseGAE(Plugin):
             self._data_path = os.path.join(tempfile.gettempdir(),
                                            'nosegae.datastore')
             self._temp_data = True
+        
+        self.sandbox_enabled = options.sandbox_enabled 
+        
         try:
+            if 'google' in sys.modules:
+                # make sure an egg (e.g. protobuf) is not cached 
+                # with the wrong path:
+                del sys.modules['google']
+            saved_path = [p for p in sys.path]
+            # import the pseudo dev_appserver (which is really a script)
+            # and let it add 3rd party libraries:
+            from dev_appserver import fix_sys_path
+            fix_sys_path() # wipes out sys.path
+            sys.path.extend(saved_path) # put back our previous path
+            
             from google.appengine.tools import dev_appserver
             from google.appengine.tools.dev_appserver_main import \
                 DEFAULT_ARGS, ARG_CLEAR_DATASTORE, ARG_LOG_LEVEL, \
@@ -80,12 +97,21 @@ class NoseGAE(Plugin):
             import django
         except ImportError, e:
             self.enabled = False
-            warn("Google App Engine not found in %s" % options.gae_lib_root,
-                 RuntimeWarning)
+            raise
+            # warn("Google App Engine not found in %s" % options.gae_lib_root,
+            #      RuntimeWarning)
         if sys.version_info[0:2] < (2,5):
             raise EnvironmentError(
                 "Python version must be 2.5 or greater, like the Google App Engine environment.  "
                 "Tests are running with: %s" % sys.version)
+        
+        # As of SDK 1.2.5 the dev_appserver.py aggressively adds some logging handlers.
+        # This removes the handlers but note that Nose will still capture logging and 
+        # report it during failures.  See Issue 25 for more info.
+        rootLogger = logging.getLogger()
+        for handler in rootLogger.handlers:
+            if isinstance(handler, logging.StreamHandler):
+                rootLogger.removeHandler(handler)
                         
     def begin(self):
         args = self._gae['DEFAULT_ARGS']
@@ -119,6 +145,11 @@ class NoseGAE(Plugin):
             sandbox_root = self._path
             testMatch = self.config.testMatch
             module_dict = self._setup_shared_modules()
+            
+            def should_sandbox(hook, *args, **kwargs):
+                if self.sandbox_enabled:
+                    return super(Hook, hook).should_sandbox(*args, **kwargs)
+        
         self.hook = Hook(sys.modules)
         sys.meta_path = [self.hook]
         # set up allowed file access paths
